/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2018 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCK_SSHAGENTHELPER_H
#define FANCYSOCK_SSHAGENTHELPER_H

#include <fancysslbase.h>

#include <fancyobject.h>
#include <fancyrunnable.h>

//#include <openssl/err.h>
//#include <openssl/pem.h>
//#include <openssl/evp.h>
#include <openssl/rsa.h>
//#include <openssl/bio.h>

//#include <iomanip>
//#include <cstring>

#ifdef WIN32
#   define UNIX_DEF_SOCK_ADDR                               "\\\\.\\pipe\\LOCAL\\fancy-ssh-agent.sock"
#else
#   define UNIX_DEF_SOCK_ADDR                               "fancy-ssh-agent.sock"
#endif


namespace Fancy {

template<class T> class ArrayObject;
class BufferChannel;
class ConfigOptions;
class List;
class ThreadPool;

namespace SSL {
class BigNum;
class PKey;
}
// this class requires FancySSL / OpenSSL stuff but is not really SSL specific, so stay without namespace SSL

namespace Sock {


/*
 * API for key containers
 */
class FANCYSSL_API SSHKeyVault : public virtual Fancy::Object
{
public:
	typedef Fancy::ObjPtr<SSHKeyVault> ptr_t;

	virtual Fancy::ObjPtr<Fancy::List> getKeyList(
			const Fancy::ObjPtr<Fancy::ConfigOptions> & confOptions = 0) = 0;

	virtual Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > signChallenge(
			const std::string & keyComment,
			uint32_t flags,
			const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > & challenge) = 0;
};


/*
 * Legacy SSHKeyVault implementation based on ~/.ssh/id_* or similar static key stores
 */
class FANCYSSL_API LegacySSHKeyVault : public virtual SSHKeyVault
{
public:
	typedef Fancy::ObjPtr<LegacySSHKeyVault> ptr_t;

	static ptr_t newInstance();

	// SSHKeyVault

	virtual Fancy::ObjPtr<Fancy::List> getKeyList(
			const Fancy::ObjPtr<Fancy::ConfigOptions> & confOptions = 0) = 0;

	virtual Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > signChallenge(
			const std::string & keyComment,
			uint32_t flags,
			const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > & challenge) = 0;

	// extras

	virtual void clearPrivKeys() = 0;

	virtual void addPrivKeyFromPEM(
			const std::string & privKeyPath) = 0;

	virtual void addPrivKeys(
			const Fancy::ObjPtr<Fancy::List> & privKeys) = 0;

	virtual void addPrivKey(
			const Fancy::ObjPtr<Fancy::SSL::PKey> & privKey) = 0;
};


class FANCYSSL_API SSHAgentHelper :
    public virtual Fancy::Object,
    public virtual Fancy::Runnable
{
public:
	typedef Fancy::ObjPtr<SSHAgentHelper> ptr_t;

	// creates an SSHAgentHelper
	static ptr_t newInstance(
			const std::string & sockAddr,
			const Fancy::ObjPtr<SSHKeyVault> & keyVault,
			const Fancy::ObjPtr<Fancy::ConfigOptions> & keylistOptions = 0);

	//virtual void setPrivKeyFactory(
	//		const Fancy::ObjPtr<SSHKeyVault> & keyVault) = 0;

	// Runnable
    virtual void run() = 0;
    virtual void requestInterruption() = 0;

	// static SSH stuff helpers

	static std::string sshReqCodeToStr(
			int8_t reqCode);

	// Converts a uint32_t to network byte order and writes it
	static void sshWriteValue(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			uint32_t val);

	// Writes the length of the data and the data itself to the specified buffer
	static void sshWriteData(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > & data);

	// Writes the length of s and s itself to the specified buffer
	static void sshWriteData(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			const std::string& s);

	// Writes the length of the BIG_NUM and the BIGNUM itself,
	// as MPINT ac. https://tools.ietf.org/html/rfc4251#section-5
	static void sshWriteBigNum(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			const BIGNUM *bn);
	        // or const Fancy::ObjPtr<Fancy::SSL::BigNum> & bn

	// Writes a public key using SSH specific format (i.e. RFC4251 and RFC4253)
	static void sshWritePubKey(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			const Fancy::ObjPtr<Fancy::SSL::PKey> & pKey);

	// Outputs the number of keys followed by the public data of the keys contained in 'keyList'
	static void sshWritePubKeyList(
			const Fancy::ObjPtr<Fancy::BufferChannel> & writeBuffer,
			const Fancy::ObjPtr<Fancy::List> &keyList);

	// Reads a uint32_t value from the given buffer at the specified offset, converts it to host byte order and returns it.
	static uint32_t sshReadValue(
			const Fancy::ObjPtr<Fancy::BufferChannel> &  readBuffer);

	// Reads a length-prefixed block of data from the given buffer
	static Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > sshReadData(
			const Fancy::ObjPtr<Fancy::BufferChannel> &  readBuffer);

	// Reads a length-prefixed BigNum from the given buffert
	static Fancy::ObjPtr<Fancy::SSL::BigNum> sshReadBigNum(
			const Fancy::ObjPtr<Fancy::BufferChannel> &  readBuffer);

};


}} // end namespaces

#endif // ! FANCYSOCK_SSHAGENTHELPER_H
