/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / ITEG
**
** A class for enrypting/decrypting data items using a symmetric key.
**
*****************************************************************************/

#ifndef FANCYSSL_CIPHER_H
#define FANCYSSL_CIPHER_H

#include <fancysslbase.h>

namespace Fancy {
namespace SSL {

/**
 *  Handles an OpenSSL CMAC for creating an AES-based cryptographic MAC.
 */
class FANCYSSL_API Cipher : virtual public Base {

  protected:
	  /**
	   * Construct a digest object taking ownership of a fully initialized
	   * digest context.
	   */
    Cipher ();

	  /**
	   * Free the associated OpenSSL context.
	   */
    virtual ~Cipher();

  public:

    enum Mode {

        /**
         * Electronic Cookbook Mode
         */
        ECB,
        /**
         * Cipher Block Chaining
         */
        CBC,
        /**
         * Output Feedback Mode
         */
        OFB
    };

    enum EncryptDecrypt {

        /**
         * Encryption Mode
         */
        Encrypt,
        /**
         * Decryption Mode
         */
        Decrypt
    };

    typedef Fancy::ObjPtr<Cipher> ptr_t;

    /**
     * Instantiate a new Cipher instance using an AES-128, AES-192 or AES-256 key.
     * If @c keylen is 16, AES-128 is used, if 24 is passed as @c keylen,
     * AES-192 is used, if 32 is passed as @c keylen, AES-256 is used.
     *
     * The given initialization vector must be of size 16 for CBC and OFB modes.
     * For ECB mode, set the initialization vector to NULL.
     *
     * @return a new instance for the given AES algorithm initialized with the given key.
     */
    static ptr_t newAESInstance(
            const void *key, size_t keylen,
            const void *iv, size_t ivlen,
            Mode mode,
            EncryptDecrypt direction);

    /**
     * Instantiate a new Cipher instance using an DES, DES-2 or DES3 key.
     * If @c keylen is 8, DES is used, if 16 is passed as @c keylen,
     * triple DES with a 128/112 bit key is used, if 32 is passed as
     * @c keylen, triple DES with a 192/168 bit key is used.
     *
     * The given initialization vector must be of size 8 for CBC and OFB modes.
     * For ECB mode, set the initialization vector to NULL.
     *
     * @return a new instance for the given DES algorithm initialized with the given key.
     */
    static ptr_t newDESInstance(
            const void *key, size_t keylen,
            const void *iv, size_t ivlen,
            Mode mode,
            EncryptDecrypt direction);

    /**
     * Instantiate a new Cipher instance using the given algorithm name.
     * The cipher mode is deduced from the algorithm name.
     *
     * The given initialization vector must be of size 8 for CBC and OFB modes.
     * For ECB mode, set the initialization vector to NULL.
     *
     * @return a new instance for the given algorithm initialized with the given key.
     */
    static ptr_t newInstance(const char *algorithmName,
            const void *key, size_t keylen,
            const void *iv, size_t ivlen,
            EncryptDecrypt direction);

    /**
     * Set whether standard PKCS#5/PKCS#7 padding is used (the standard setting) or
     * no padding is used and the encrypted data is either a multiple of the block
     * size or the upstream application handles padding on its own.
     */
    virtual void setPadding(bool padding) = 0;

    /**
     * @return Whether PKCS#5/PKCS#7 padding is enabled or not.
     */
    virtual bool isPadding() const =0;

    /**
     * @return The block size of the underlying cipher.
     */
    virtual size_t getBlockSize() const = 0;

    /**
     * Update the digest with @c inLen bytes of the given binary data
     * in @c in and write intermediate results to @c out.
     *
     * The length @c outLen of @c out should be at least @c inLen plus the
     * block size as returned by @c getBlockSize().
     *
     * After updating the cipher with input data, you have to finally call
     * @c finalize() in order to success
     *
     * @return The number of bytes written to @c out, which may also be zero.
     */
    virtual size_t update(const void *in, size_t inLen, void *out, size_t outLen) = 0;

    /**
     * Write the final block of encrypted or decrypted data to
     * out. If outLen is smaller than the block size returned by
     * @c getBlockSize() an exception is thrown.
     *
     * In encryption mode an exception will be thrown if padding is
     * disabled and the final block is not a full multiple of the block
     * size.
     *
     * In decryption mode an exception will be thrown if padding is
     * enabled and the final block is not correctly formatted.
     *
     * The digest object is unusable after this operation has been
     * performed.
     *
     * @return The number of bytes written to @c out, which may be zero.
     */
    virtual size_t finalize(void *out, size_t outLen) = 0;

  private:
    Cipher (const Cipher &a);
    Cipher &operator= (const Cipher &a);
};

}}

#endif
