/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / ITEG
**
** A class for creating message authentication keys (MACs).
**
*****************************************************************************/

#ifndef FANCYSSL_CMAC_H
#define FANCYSSL_CMAC_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
struct CMAC_CTX_st;
typedef struct CMAC_CTX_st CMAC_CTX;
};

namespace Fancy {
namespace SSL {

/**
 *  Handles an OpenSSL CMAC for creating an AES-based cryptographic MAC.
 */
class FANCYSSL_API CMAC : public Base {

  protected:
	  /**
	   * Construct a digest object taking ownership of a fully initialized
	   * digest context.
	   */
    CMAC (CMAC_CTX *_ctx);

	  /**
	   * Free the associated OpenSSL context.
	   */
    virtual ~CMAC();

  public:

	  typedef Fancy::ObjPtr<CMAC> ptr_t;

	  /**
	   * Instantiate a new CMAC instance using an AES_128 or AES-256 key.
	   * If @c keylen is 16, AES-128 is used, if 32 is passed as @c keylen,
	   * AES-256 is used.
	   *
	   * @return a new instance for the given AES algorithm initialized with the given key.
	   */
      static ptr_t newInstance(const void *key, size_t keylen);

      /**
       * Update the digest with the given binary data.
       */
      void addMessage(const void *m, size_t len);

      /**
       * @return The length of the generated digests.
       */
      size_t getMACSize() const;

      /**
       * Write the value of the digest to the memory location
       * pointed to by @c out with size @c outLen.
       *
       * If outLen is smaller than the digest size returned by
       * @c getMACSize() an exception is thrown.
       *
       * The digest object is cleared and it may be reused by calling
       * @c add*() and @getDigest() again.
       *
       * @return The number of bytes written to @c out.
       */
      size_t getMAC(void *out, size_t outLen) const;

    private:
      CMAC (const CMAC &a);
      CMAC &operator= (const CMAC &a);

      CMAC_CTX *ctx;
    };
  }
}

#endif
