/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / ITEG
**
** A class for implementing a custom RSA key.
**
*****************************************************************************/

#ifndef FANCYSSL_CUSTOM_RSA_H
#define FANCYSSL_CUSTOM_RSA_H

#include <fancysslbase.h>

#include <string>


// fwd decls from openssl.
extern "C" {
struct rsa_st;
typedef struct rsa_st RSA;
}

namespace Fancy {

template<class T> class ArrayObject;

namespace SSL {

class PKey;

/**
 * A representation of an OpenSSL SSL private key with custom
 * method implementations, which implements
 */
class FANCYSSL_API CustomRSA : public Base
{
protected:

	CustomRSA();
	~CustomRSA();
public:

	typedef Fancy::ObjPtr<CustomRSA> ptr_t;

	/**
	 * @param rsa An implementation of the CustomRSA interface. The OpenSSL method will
	 *            be chosen based on the fact whether the passed instance
	 *            additionally implements @c CustomRSAPad or not.
	 * @return A new PKey instance to use for a SSL context or the like.
	 */
	static Fancy::ObjPtr<PKey> newPKeyInstance(const Fancy::ObjPtr<CustomRSA> &rsa);

	/**
	 * Returns size of the RSA key in bytes, if it is an RSA key. -1 otherwise.
	 */
	virtual int getRSASize() const = 0;

	/**
	 * @return A public key RSA instance, which is managed by this object itself.
	 *         The caller must not call RSA_free() on the returned instance.
	 */
	virtual RSA *getPublicKey() const = 0;

	/**
	 * The low-level private decryption operation.
	 *
	 * Note: implementations might use @c RSA_padding_check_PKCS1_type_1() in order to
	 * remove PKCS#1 padding if padding == RSA_PKCS1_PADDING.
	 *
	 * @param flen the length of the data stored in @c from to be decrypted, which is usually @c getRSASize().
	 * @param from A pointer to the data to be decrypted.
	 * @param to A pointer to the resulting encrypted data,
	 *           which must have the length of @c getRSASize().
	 * @param padding One of RSA_PKCS1_PADDING, RSA_PKCS1_OAEP_PADDING,RSA_SSLV23_PADDING or RSA_NO_PADDING,
	 *                see @c RSA_PUBLIC_ENCRYPT(3SSL) for details.
	 * @return The size of the decrypted data. In case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual unsigned int privateDecrypt(int flen, const unsigned char *from,
                                unsigned char *to, int padding) const = 0;

	/**
	 * The low-level public encryption operation.
	 *
	 * The default implementation performs a public encrpyt operation using the public key returned
	 * by @c getPublicKey().
	 *
	 * @param flen the length of the data stored in @c from to be encrypted.
	 * @param from A pointer to the data to be decrypted.
	 * @param to A pointer to the resulting encrypted data,
	 *           which must have the length of @c getRSASize().
	 * @param padding One of RSA_PKCS1_PADDING, RSA_PKCS1_OAEP_PADDING,RSA_SSLV23_PADDING or RSA_NO_PADDING,
	 *                see @c RSA_PUBLIC_ENCRYPT(3SSL) for details.
	 * @return The size of the decrypted data, which is usually @c getRSASize().
	 *         In case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual unsigned int publicEncrypt(int flen, const unsigned char *from,
			                   unsigned char *to, int padding) const;

	/**
	 * The low-level private encryption operation, which is used for raw signatures.
	 *
	 * Note: implementations might use @c RSA_padding_add_PKCS1_type_1() in order to
	 * introduce PKCS#1 padding if padding == RSA_PKCS1_PADDING.
	 *
	 * @param flen the length of the date stored in @c from to be encrypted.
	 * @param from A pointer to the data to be encrypted, usually a message digest with
	 *             a digest algorithm identifier.
	 * @param to A pointer to the resulting encrypted data,
	 *           which must have the length of @c getRSASize().
	 * @param padding Either RSA_PKCS1_PADDING or RSA_NO_PADDING,
	 *                see @c RSA_PRIVATE_ENCRYPT(3SSL) for details.
	 * @return The size of the encrypted data aka the signature, which is usually @c getRSASize().
	 *         In case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual unsigned int privateEncrypt(int flen, const unsigned char *from,
                                unsigned char *to, int padding) const = 0;

	/**
	 * The low-level public decryption operation, which is used to verify raw signatures.
	 *
	 * The default implementation performs a public encrypt operation using the public key returned
	 * by @c getPublicKey().
	 *
	 * @param flen the length of the date stored in @c from to be decrypted, which is usually @c getRSASize().
	 * @param from A pointer to the data to be decrypted, usually a signature of size @c getRSASize().
	 * @param to A pointer to the resulting encrypted data,
	 *           which must have the length of @c getRSASize().
	 * @param padding Either RSA_PKCS1_PADDING or RSA_NO_PADDING,
	 *                see @c RSA_PRIVATE_ENCRYPT(3SSL) for details.
	 * @return The size of the decrypted data aka the payload of a signature
	 *         composed of a digest algorithm OID and the digest value.
	 *         In case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual unsigned int publicDecrypt(int flen, const unsigned char *from,
                               unsigned char *to, int padding) const;

	/**
	 * @return the optional key comment.
	 */
	virtual std::string getKeyComment() const = 0;

private:
	CustomRSA (const CustomRSA &a);
	CustomRSA &operator= (const CustomRSA &a);
};

/**
 * A representation of an OpenSSL SSL private key with custom
 * method implementations, which provide signature padding on their own
 * and may not provide for the raw signature implementations
 * @c privateEncrypt() and @c publicDecrypt().
 *
 * You rarely will need this interface, most signature engines will
 * implement CustomRSA::privateEncrypt().
 */
class FANCYSSL_API CustomRSAPad: virtual public CustomRSA
{
protected:

	CustomRSAPad();
	~CustomRSAPad();
public:

	typedef Fancy::ObjPtr<CustomRSAPad> ptr_t;

	/**
	 * Sign a message digest of a given digest algorithm and assembles the
	 * likewise signature content (algorithm identifier plus digest plus PKCS#1 padding).
	 *
	 * @param type The NID of the algorithm of the digest, which e.g. may be NID_sha256, NID_md5_sha1 or NID_sha.
	 * @param m The buffer containing the message digest.
	 * @param m_length The length of the input buffer, which must match the length of the
	 *                 digests create by the given digest algorithm. (e.g. 32 for NID_sha256).
	 * @param sigret A pointer to the resulting signature,
	 *           which must have the length of @c getRSASize().
	 * @return The size of the resulting signature, which is usually @c getRSASize().
	 *         In case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual unsigned int sign(int type,
            const unsigned char *m, unsigned int m_length,
            unsigned char *sigret) const = 0;

	/**
	 * Verify a signed message digest of a given digest algorithm and checks the
	 * likewise signature content (algorithm identifier plus digest plus PKCS#1 padding).
	 *
	 * The default implementation performs a verification operation using the public key returned
	 * by @c getPublicKey().
	 *
	 * @param type The NID of the algorithm of the digest, which e.g. may be NID_sha256, NID_md5_sha1 or NID_sha.
	 * @param m The buffer containing the message digest.
	 * @param m_length The length of the input buffer, which must match the length of the
	 *                 digests create by the given digest algorithm. (e.g. 32 for NID_sha256).
	 * @param sigbuf A pointer to the signature to be verified,
	 *           which usually has a length of @c getRSASize().
	 * @param siglen The length of the signatur contained in @c sigbuf, usually @c getRSASize().
	 * @throws in case of any error, an exception must be thrown,
	 *         which will be set to the openssl error state.
	 */
	virtual void verify(int dtype,
			const unsigned char *m, unsigned int m_length,
			const unsigned char *sigbuf, unsigned int siglen) const;

private:
	CustomRSAPad (const CustomRSAPad &a);
	CustomRSAPad &operator= (const CustomRSAPad &a);
};

}
}

#endif
