/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
 **
 ** Author: Christoph Lechleitner / ITEG
 **
 ** A class for handling DES keys.
 **
 *****************************************************************************/

#ifndef FANCYSSL_DESKEY_H
#define FANCYSSL_DESKEY_H

#include <fancysslbase.h>

#include <openssl/des.h>

// fwd decls from openssl.
//extern "C" {
//  struct evp_pkey_st;
//  typedef struct evp_pkey_st EVP_PKEY;
//};

namespace Fancy {

template<class T> class ArrayObject;

namespace SSL {

/**
 * A representation of a DES key.
 */
class FANCYSSL_API DESKey : public Base
{
protected:
	DESKey();
	~DESKey();

public:

	typedef Fancy::ObjPtr<DESKey> ptr_t;

	/**
	 * Creates a new, invalid DESKey.
	 * Use setRandom() or setKey() to fill it.
	 */
	static ptr_t newInstance();

	/**
	 * Creates a new DESKey and initializes it with the first 8 bytes of keySet (uses setKey()).
	 * If keySet is null or too small, an invalid key is returned
	 */
	static ptr_t newInstance(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & keySet,
			bool check=false);

	/**
	 * Creates a new DESKey with randomized content (uses setRandom())
	 */
	static ptr_t newRandomInstance();

	/**
	 * Returns the keySet (mirrors DES_key_schedule.DES_cblock)
	 */
	Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > getKeySet();

	/**
	 * Returns the keySet (mirrors DES_key_schedule.DES_cblock)
	 */
	DES_key_schedule * getKeySchedule();

	/**
	 * Tells it the internal data look valid
	 */
	bool isValid();

	/**
	 * Initialized this DES key object from randomly generated 8 bytes passed
	 * in by \c key. The passed random number is copied and the parity is implicitly
	 * set to odd before actually initializing the key.
	 *
	 * If \c check is true, it is checked, whether the key is weak.
	 *
	 * For Details on key checking see \c DES_crypt(3).
	 */
	bool setKey(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & keySet,
			bool check=false);

	/**
	 * Initialized this DES key with a crytographically strong random number.
	 *
	 * Return false, if the random number generator is not seeded enough.
	 *
	 * For Details on openSSL random number generation see
	 * \c DES_random_key(3) and \c RAND_bytes(3).
	 */
	bool setRandom();

	/**
	 * Performs a 3-DES EDE CBC encryption using three keys. If the 8-byte
	 * initialization vector \c ivec is a null pointer, an initialization vector
	 * consisting of 8 zero bytes is used instead.
	 *
	 * Warning: The input is padded with zeros, if the length is not a multiple of 8.
	 * The length of the output is always a multiple of 8. So in order to
	 * exactly restore the original data using \c ede3cbcDecrypt(), you are
	 * responsible for storing the length of the data or to use a padding-insensitive
	 * encoding of your data.
	 *
	 * If any of the three keys passed is invalid, emtpy array is returned.
	 *
	 * For Details on the encryption see \c DES_crypt(3).
	 */
	static Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > ede3cbcEncrypt(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > &clearTextData,
			const Fancy::ObjPtr<DESKey> &k1,
			const Fancy::ObjPtr<DESKey> &k2,
			const Fancy::ObjPtr<DESKey> &k3,
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & ivec = 0);

	/**
	 * Performs a 3-DES EDE CBC decryption using three keys. If the 8-byte
	 * initialization vector \c ivec is a null pointer, an initialization vector
	 * consisting of 8 zero bytes is used instead.
	 *
	 * If any of the three keys passed is invalid, emtpy array is returned.
	 *
	 * For Details on the encryption see \c DES_crypt(3).
	 */
	static Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > ede3cbcDecrypt(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > &encryptedData,
			const Fancy::ObjPtr<DESKey> &k1,
			const Fancy::ObjPtr<DESKey> &k2,
			const Fancy::ObjPtr<DESKey> &k3,
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & ivec = 0);


	/**
	 * Performs a 3-DES EDE CBC encryption using two keys. If the 8-byte
	 * initialization vector \c ivec is a null pointer, an initialization vector
	 * consisting of 8 zero bytes is used instead.
	 *
	 * If any of the two keys passed is invalid, emtpy array is returned.
	 *
	 * Warning: The input is padded with zeros, if the length is not a multiple of 8.
	 * The length of the output is always a multiple of 8. So in order to
	 * exactly restore the original data using \c ede2cbcDecrypt(), you are
	 * responsible for storing the length of the data or to use a padding-insensitive
	 * encoding of your data.
	 *
	 * For Details on the encryption see \c DES_crypt(3).
	 */
	static Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > ede2cbcEncrypt(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > &clearTextData,
			const Fancy::ObjPtr<DESKey> &k1,
			const Fancy::ObjPtr<DESKey> &k2,
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & ivec = 0);

	/**
	 * Performs a 3-DES EDE CBC decryption using two keys. If the 8-byte
	 * initialization vector \c ivec is a null pointer, an initialization vector
	 * consisting of 8 zero bytes is used instead.
	 *
	 * If any of the two keys passed is invalid, emtpy array is returned.
	 *
	 * For Details on the encryption see \c DES_crypt(3).
	 */
	static Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > ede2cbcDecrypt(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > &encryptedData,
			const Fancy::ObjPtr<DESKey> &k1,
			const Fancy::ObjPtr<DESKey> &k2,
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & ivec = 0);

	/**
	 * Performs a CBC-MAC-chain-computation, hopefully implementing X9.19
	 *
	 * If any of the two keys passed is invalid, emtpy array is returned.
	 *
	 * For Details on the encryption see \c des(3).
	 */
	static Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > computeCbcMac(
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > &data,
			const Fancy::ObjPtr<DESKey> &k1,
			const Fancy::ObjPtr<DESKey> &k2,
			const Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > & ivec = 0);
private:

	DES_key_schedule *keySchedule;
	Fancy::ObjPtr<Fancy::ArrayObject <int8_t> > keySet;

};

}} // end namespaces

#endif // FANCYSSL_DESKEY_H
