/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for creating message digests.
**
*****************************************************************************/

#ifndef FANCYSSL_DIGEST_H
#define FANCYSSL_DIGEST_H

#include <fancysslbase.h>
#include <fancyarray.h>


// fwd decls from openssl.
extern "C" {
	#include <openssl/opensslv.h>

// 'env_md_ctx_st' and 'env_md_st' have been renamed in OpenSSL 1.1
// to 'evp_md_ctx_st' and 'evp_md_st'
#if OPENSSL_VERSION_NUMBER < 0x10100000L
	struct env_md_ctx_st;
	typedef struct env_md_ctx_st EVP_MD_CTX;
	struct env_md_st;
	typedef struct env_md_st EVP_MD;
#else
	struct evp_md_ctx_st;
	typedef struct evp_md_ctx_st EVP_MD_CTX;
	struct evp_md_st;
	typedef struct evp_md_st EVP_MD;
#endif

}

namespace Fancy {

  class Channel;

  namespace SSL {

  /**
   *  Handles an OpenSSL BIO Structure for creating a message digest.
  */
  class FANCYSSL_API Digest : public Base
  {

  protected:
	  /**
	   * Construct a digest object taking ownership of a fully initialized
	   * digest context.
	   */
	  Digest (EVP_MD_CTX *_ctx, const EVP_MD *_ctx_digest);

	  /**
	   * Free the associated OpenSSL context.
	   */
	  ~Digest();

  public:

	  typedef Fancy::ObjPtr<Digest> ptr_t;

	  /**
	   * Supported algorithm names are "SHA1", "SHA256" etc...
	   *
	   * @return a new instance for the given algorithm name.
	   */
      static ptr_t newInstance(const char *algo);

      /**
       * Update the digest with the given binary data.
       */
      void addMessage(const uint8_t *m, size_t len);

      /**
       * Update the digest with all remaining data from the given I/O channel.
       */
      void addChannel(const Fancy::ObjPtr<Fancy::Channel> &channel);

      /**
       * Update the digest with all data from the given file.
       */
      void addFile(const char *filename);

      /**
       * @return The length of the generated digests.
       */
      size_t getDigestSize() const;

      /**
       * return the value of the digest as an upper case hexadecimal string.
       * The digest object is cleared and it may be reused by calling
       * @c add*() and @getDigest() again.
       */
      std::string getHexDigest() const;

      /**
       * Return the value of the digest as a byte array.
       * The digest object is cleared and it may be reused by calling
       * @c add*() and @getDigest() again.
       */
      Fancy::ArrayObject<uint8_t>::ptr_t getDigest() const;

      /**
       * Write the value of the digest to the memory location
       * pointed to by @c out with size @c outLen.
       *
       * If outLen is smaller than the digest size returned by
       * @c getDigestSize() an exception is thrown.
       *
       * The digest object is cleared and it may be reused by calling
       * @c add*() and @getDigest() again.
       *
       * @return The number of bytes written to @c out.
       */
      size_t getDigest(void *out, size_t outLen) const;

      /**
       * Return the result of @c getHexDigest().
       */
      std::string toString() const;

    private:
      Digest (const Digest &a);
      Digest &operator= (const Digest &a);

      EVP_MD_CTX *ctx;
      const EVP_MD *ctx_digest;
    };
  }
}

#endif
