/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2019 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
 **
 ** Author: Wolfgang Glas / iteg
 **
 ** A class for handling OpenSSL engines.
 **
 *****************************************************************************/

#ifndef FANCYSSL_ENGINE_H
#define FANCYSSL_ENGINE_H

#include <fancysslbase.h>

#include <string>


// fwd decls from openssl.
extern "C" {
struct engine_st;
typedef struct engine_st ENGINE;
}

namespace Fancy {
namespace SSL {

class PKey;

/**
 * A representation of an OpenSSL private key.
 */
class FANCYSSL_API Engine : public Base
{
protected:
	Engine (ENGINE *e);

	~Engine();
public:
	typedef Fancy::ObjPtr<Engine> ptr_t;

	/**
	 * Acquires the given OpenSSL handle and calls ENGINE_free() in the destructor.
	 *
	 * You must once call initialize() on the returned object in order to actually use
	 * the engine.
	 */
	static ptr_t newInstance(ENGINE *e);

	/**
	 * Search for an already registered engine with the given ID.
	 *
	 * This method throws an exception, if no registered engine for the given ID can be found.
	 * *
	 * You must once call initialize() on the returned object in order to actually use
	 * the engine.
	 *
	 * @param engine_id  The ID of the engine like "pkcs11".
	 * @return The registered engine.
	 */
	static ptr_t getByID(const char *engine_id);

	/**
	 * Search for an already registered engine with the given ID or dynamically load the
	 * given engine as per the dynamic_path configuration in the likewise configuration
	 * section for the given engine.
	 *
	 * This method throws an exception, if the engine cannot be loaded.
	 *
	 * You must once call initialize() on the returned object in order to actually use
	 * the engine.
	 *
	 * @param engine_id  The ID of the engine like "pkcs11".
	 * @return The registered or newly loaded engine.
	 */
	static ptr_t getOrLoadByID(const char *engine_id);

	/**
	 * Returns the internal ENGINE pointer
	 */
	ENGINE* getENGINE() const;

	/**
	 * Returns the ID of this engine like "pkcs11".
	 */
	const char *getID() const;

	/**
	 * Acquires a functional reference to the engine to actually use it.
	 * The functional reerence will be dropped by finish() or by the destructor.
	 */
	void initialize();

	/**
	 * Releases the functional reference of the referenced engine.
	 */
	void finish();

	/**
	 * Returns true if initialize() has been called on this object.
	 * false will be returned after calling finish().
	 */
	bool isInitialized() const;

	/**
	 * Set this engine as default engine for the given set of algorithms as
	 * specified for the given BIT mask composed of ENGINE_METHOD_* defines
	 * in opensl/engine.h
	 */
	void setDefault(unsigned flags) const;

	/**
	 * Load a private key using the given key ID, which may contain a string
	 * like "pkcs11:pin-value=123456".
	 *
	 * @return a private key object.
	 * @throws An exception, if the key could not be loaded.
	 */
	Fancy::ObjPtr<PKey> loadPrivateKey(const char *key_id) const;

private:
	Engine (const Engine &a);
	Engine &operator= (const Engine &a);

	ENGINE *engine;
	bool initialized;
};

}} // end namespaces

#endif
