/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/


#ifndef FANCYSSL_OAEP_H
#define FANCYSSL_OAEP_H

#include <fancysslbase.h>
#include <fancyarray.h>

namespace Fancy {
namespace SSL {

class Digest;
class MGF;
class Random;

/**
 * OAEP algorithm as defined in
 * <a href="http://tools.ietf.org/html/rfc3447#section-7.1">Section 7.1 of RFC 3447</a>.
 */
class FANCYSSL_API OAEP : virtual public Fancy::Object
{

protected:
	OAEP();
	virtual ~OAEP();

public:

	typedef Fancy::ObjPtr<OAEP> ptr_t;

	/**
	 * Return a new instance implementing
	 * the mask generation function no. 1 for the OAEP algorithm as defined in
	 * <a href="http://tools.ietf.org/html/rfc3447#appendix-B.2">Appendix B.2.1 of RFC 3447</a>.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<Digest> &digest,
	        const Fancy::ObjPtr<MGF> &mgf,
	        const Fancy::ObjPtr<Random> &random);

    /**
     * Prepare the given input data for RSA encryption.
     *
     * @param in The input buffer
     * @param inLen The length of the data to be encrypted.
     * @param label The encryption label used as additional seed material.
     *              May be NULL, if labelLen is zero.
     * @param labelLen The size of the encryption label.
     * @param out The output buffer. The length of the output buffer is the
     *            size of the RSA modulus used.
     * @param outLen The size of the output to be generated, usually the size
     *               of the RSA label.
     */
    virtual void encrypt(const void *in, size_t inLen,
            const void *label, size_t labelLen,
            void *out, size_t outLen) const = 0;

    /**
     * Prepare the given input data for RSA encryption.
     *
     * @param in The input buffer, which not longer than <code>keyLen</code>.
     * @param inLen The length of the data to be decrypted.
     * @param keyLen The length of the RSA modulus. The length of <code>in</code>
     *               may be less than <code>keyLen</code>, because <code>in</code>
     *               is the magnitude of a big integer, which might have contained
     *               leading zero bytes. Hence, if <code>in</code> is shorter than
     *                <code>keyLen</code>, the missing parts are interpreted as
     *               leading zero bytes.
     * @param label The encryption label used as additional seed material.
     * @param labelLen The size of the encryption label.
     * @param out The output buffer. The length of the output buffer must be at least
     *            the size of the RSA modulus used.
     * @param outOff The offset at which the result is stored in <code>out</code>.
     *
     * @return A byte array of decrypted content.
     */
    virtual Fancy::ArrayObject<uint8_t>::ptr_t decrypt(
            const void *in, size_t inLen,
            size_t keyLen,
            const void *label, size_t labelLen) const = 0;

    /**
     * @return The message digest, which is used by this OAEP instance.
     */
    virtual Fancy::ObjPtr<Digest> getDigest() const = 0;

    /**
     * @return The mask generation function, which is used by this OAEP instance.
     */
    virtual Fancy::ObjPtr<MGF> getMGF() const = 0;

    /**
     * @return The random number generator, which is used by this OAEP instance.
     */
    virtual Fancy::ObjPtr<Random> getRandom() const = 0;
};
}
}

#endif
