/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling PKCS7 signatures and messages.
**
*****************************************************************************/

#ifndef FANCYSSL_PKCS7MSG_H
#define FANCYSSL_PKCS7MSG_H

#include <fancysslbase.h>
#include <fancylist.h>
#include <time.h>

// fwd decls from openssl.
extern "C" {
  struct pkcs7_st;
  typedef struct pkcs7_st PKCS7;
};

namespace Fancy {
  namespace SSL {
    class X509Cert;
    class PKey;

    class FANCYSSL_API PKCS7Msg : public Base
    {

    protected:
      PKCS7Msg (PKCS7 *_pkcs7);

      ~PKCS7Msg();

    public:
    	typedef Fancy::ObjPtr<PKCS7Msg> ptr_t;

    	/**
    	 * Acquire the given OpenSSL pointer, which freed, when the returned object goes
    	 * out of scope.
    	 *
    	 * @param pkcs7 An OpenSSl pkcs7 structure.
    	 * @return A new @c PKCS7Msg instance, which manages the given OpenSSL pointer.
    	 */
    	static ptr_t newInstance(PKCS7 *pkcs7);

    	/**
          Reads the PKCS7 structure from a PEM-encoded string.

          @param pem The PEM-encoded string.
          @param len if set to-1 then then @c pem is assumed to be null terminated and its
                      length is determined by strlen.
    	  @return A new PKCS7Msg instance.
        */
    	static ptr_t newInstance(const char *pem, int len=-1);

        /*!
          Returns, whether this object represents a PKCS7 detached signature.
          If this is true, you have to call @c verifiyDetached() instead of
          @c verifyAttached().
        */
    	bool isDetached() const;

    	/**
    	 * @return A PEM-encoded representation of this PKCS7 structure.
    	 */
     	std::string getPEMString () const;

        /*!
           Verifies the message inside the given PKCS7 structure or the  message
           supplied through \c setMessage(), if the PKCS7 structure contains a
           detached signature.

           The signer certificate is specified as an argument and is verified
           against the given CA, checks against multiple CAs are unsupported.

           Returns, whether the verification was successful.
         */

     	/**
     	 * Verifies the message inside the given PKCS7 structure.
     	 *
     	 * @param signer The expected signer certificate.
     	 * @param ca_store The CA store used to verify the signer certificate.
     	 * @param when The point in time, when the signature has been created.
     	 * @return The verified message as contained in the PKCS structure.
     	 */
    	std::string verifyAttached(const Fancy::ObjPtr<X509Cert> &signer,
    			const Fancy::ObjectArray<X509Cert>::ptr_t &ca_store,
    			time_t when=0);

     	/**
     	 * Verifies the supplied message against the given PKCS7 structure.
     	 *
     	 * @param message The message to be verified.
     	 * @param signer The expected signer certificate.
     	 * @param ca_store The CA store used to verify the signer certificate.
     	 * @param when The point in time, when the signature has been created.
     	 */
    	void verifyDetached(const std::string &message,
    			const Fancy::ObjPtr<X509Cert> &signer,
    			const Fancy::ObjectArray<X509Cert>::ptr_t &ca_store,
    			time_t when=0);

    	/**
    	 * Signs the supplied message and builds a new pkcs7
         * structure including the created signature.
         *
    	 * @param message The message to sign.
    	 * @param signer_key The private key of the signer.
    	 * @param signer The certificate of the signer.
    	 * @param additional_certs Additional certificates to add to the
    	 *               generated PKCS7 structure.
    	 * @param detached If @true, the message itself will not be part
    	 *               of the generated PKCS7 structure.
    	 * @return a new @c PKCS7Msg instance, which contains the requested signature.
    	 */
    	static ptr_t sign(const std::string &message,
    			const Fancy::ObjPtr<PKey> &signer_key,
    			const Fancy::ObjPtr<X509Cert> &signer,
    			const Fancy::ObjectArray<X509Cert>::ptr_t &additional_certs,
    			bool detached);

    	/**
    	 *
    	 * @return The OpenSSL PKCS7 pointer. The caller must not call @c PKCS7_free() on the
    	 *         returned pointer.
    	 */
    	PKCS7 *getPKCS7() const;

    private:
    	PKCS7Msg (const PKCS7Msg &a);
    	PKCS7Msg &operator= (const PKCS7Msg &a);

        PKCS7 *pkcs7;
    };
  }
}

#endif
