/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL keys.
**
*****************************************************************************/

#ifndef FANCYSSL_PKEY_H
#define FANCYSSL_PKEY_H

#include <fancysslbase.h>

#include <string>


// fwd decls from openssl.
extern "C" {
  struct evp_pkey_st;
  typedef struct evp_pkey_st EVP_PKEY;
  struct rsa_st;
  typedef struct rsa_st RSA;

}

namespace Fancy {

template<class T> class ArrayObject;

  namespace SSL {

    /**
     * A representation of an OpenSSL private key.
     */
    class FANCYSSL_API PKey : public Base
    {
    protected:
    	PKey (EVP_PKEY *k);

    	~PKey();
    public:

    	typedef Fancy::ObjPtr<PKey> ptr_t;

        /**
          Aquires the given OpenSSL handle and automagically frees the handle,
          if the object goes out of scope. The caller must not call \c PKEY_free
          on the supplied resource.
        */
    	static ptr_t newInstance(EVP_PKEY *k);

    	/**
    	 * Read a private key from a PEM-encoded string.
    	 * @param passwd The password used to decode the string or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param pem The PEM string.
    	 * @param len The length of @c pem or -1,  if the string is null-terminated.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newInstance(const char *passwd, const char *pem, int len=-1);

    	/**
    	 * Read a public key from a PEM-encoded string.
    	 * @param passwd The password used to decode the string or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param pem The PEM string.
    	 * @param len The length of @c pem or -1,  if the string is null-terminated.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newPublicInstance(const char *passwd, const char *pem, int len=-1);

    	/**
    	 * Contruct a new RSA instance.
    	 * @param rsa A pointer to an openssl RSA structure.
    	 *            The caller is responsible to free the passed RSA structure.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newRSAInstance(RSA *rsa);

    	/**
    	 * Read an RSA private key from a ASN1 block.
    	 * @param asn1 The ASN1 block.
    	 * @param len  The length of @c asn1.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newRSAInstance(const unsigned char *asn1, int len);

    	/**
    	 * Read a public key from a ASN1 block (PKCS#1 RSAPublicKey structure).
    	 * @param asn1 The ASN1 block.
    	 * @param len  The length of @c asn1.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newPublicRSAInstance(const unsigned char *asn1, int len);

    	/**
    	 * Read a public key from a modulus and public exponent.
    	 * @param modulus The modulus.
    	 * @param publicExponent  The public exponent1.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t newPublicRSAInstance(
    			const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > & modulus,
    			const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > & publicExponent);

    	/**
    	 * Read a private key from a PEM-encoded file.
    	 * @param passwd dThe password used to decode the file or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param file The path to the PEM-encoded file.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t readFile(const char *file, const char *passwd);

    	/**
    	 * Read a public key from a PEM-encoded file.
    	 * @param passwd dThe password used to decode the file or @c NULL, if the
    	 *               the string is expected to be unencrypted.
    	 * @param file The path to the PEM-encoded file.
    	 * @return A new @c PKey instance.
    	 */
    	static ptr_t readFilePublic(const char *file, const char *passwd);

    	/**
    	 * Generate an RSA private key using a given number of bits.
    	 * @param nbits the number of bits for the generated key.
    	 * @return A new @PKey instance.
    	 */
    	static ptr_t generateRSA(int nbits);

    	/**
    	 * Returns the internal EVP_PKEY pointer
    	 */
    	EVP_PKEY* getPKEY() const;

    	/**
    	 * Returns true if EVP_PKEY is set and it'ss type field says it's RSA key
    	 */
    	bool isRSA() const;

    	/**
    	 * Returns the EVP_PKEY's RSA* field if isRSA() is true
    	 */
    	RSA* getRSA() const;

    	/**
    	 * PEM-enccode this key.
    	 * @param passwd A password to encode the key or @c NULL, if the key
    	 *               should not be encoded.
    	 * @return A PEM-encoded representation of this key.
    	 */
    	std::string getPEMString(const char *passwd) const;

    	/**
    	 * PEM-enccode this public key or the public part of this private key.
    	 * @return A PEM-encoded representation of this public key.
    	 */
    	std::string getPublicPEMString() const;

    	/*
    	 * If getPEMString(passwd) is not empty, writes it to filename.
    	 * Returns wether something was written succesfully.
    	 */
    	bool writePEMString(const char *passwd, const std::string & filename);

    	/*
    	 * If getPublicPEMString() is not empty, writes it to filename.
    	 * Returns wether something was written succesfully.
    	 */
    	bool writePublicPEMString(const std::string & filename);

    	/**
    	 * Returns size of RSA key in bytes, if it is an RSA key. -1 otherwise.
    	 */
    	int getRSASize() const;

    	/**
    	 * Returns the key comment
    	 */
    	std::string getKeyComment() const;

    	/**
    	 * Sets the key comment
    	 */
    	void setKeyComment(const std::string & keyComment);

    private:
    	PKey (const PKey &a);
    	PKey &operator= (const PKey &a);

        EVP_PKEY *pkey;
        std::string keyComment; // i.e. file name, database id, ...
    };
  }
}

#endif
