/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/


#ifndef FANCYSSL_PSS_H
#define FANCYSSL_PSS_H

#include <fancysslbase.h>
#include <fancyarray.h>

namespace Fancy {
namespace SSL {

class Digest;
class MGF;
class Random;

/**
 * PSS algorithm as defined in
 * <a href="http://tools.ietf.org/html/rfc3447#section-7.1">Section 7.1 of RFC 3447</a>.
 */
class FANCYSSL_API PSS : virtual public Fancy::Object
{

protected:
	PSS();
	virtual ~PSS();

public:

	typedef Fancy::ObjPtr<PSS> ptr_t;

	/**
	 * Return a new instance implementing
	 * the mask generation function no. 1 for the PSS algorithm as defined in
	 * <a href="http://tools.ietf.org/html/rfc3447#appendix-B.2">Appendix B.2.1 of RFC 3447</a>.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<Digest> &digest,
	        const Fancy::ObjPtr<MGF> &mgf,
	        const Fancy::ObjPtr<Random> &random);

    /**
     * Prepare the given input data for RSA signature.
     *
     * @param in The input buffer to be signed.
     * @param inLen The length of the data to be signed.
     * @param sLen The size of the seed data to be included.
     * @param em The output buffer. The length of the output buffer is
     *            (emBits+7)/8.
     * @param emBits The number of bits of the output to be generated, usually the
     *               bit length of the RSA modulus minus one.
     */
    virtual void encode(const void *in, size_t inLen,
            size_t sLen,
            void *em, size_t emBits) const = 0;

    /**
     * Validate the given input data against the raw RSA encrypted signature.
     *
     * @param in The input buffer to be verified.
     * @param inLen The length of the data to be verified.
     * @param em The encoded message input buffer. The length is
     *            (emBits+7)/8.
     * @param emBits The number of bits of the encoded message to be validated,
     *               usually the bit length of the RSA modulus minus one.
     *
     * @return Whether the given encoded message is correct.
     */
    virtual bool verify(
            const void *in, size_t inLen,
            const void *em, size_t emBits) const = 0;

    /**
     * @return The message digest, which is used by this PSS instance.
     */
    virtual Fancy::ObjPtr<Digest> getDigest() const = 0;

    /**
     * @return The mask generation function, which is used by this PSS instance.
     */
    virtual Fancy::ObjPtr<MGF> getMGF() const = 0;

    /**
     * @return The random number generator, which is used by this PSS instance.
     */
    virtual Fancy::ObjPtr<Random> getRandom() const = 0;
};
}
}

#endif
