/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libfancy.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSSL_SOCK_DEVICE_H
#define FANCYSSL_SOCK_DEVICE_H

#include <fancysockdevice.h>
#include <fancysslbase.h>

extern "C"
{
  struct ssl_ctx_st;
  typedef struct ssl_ctx_st SSL_CTX;

  struct ssl_st;
  typedef struct ssl_st SSL;
}

namespace Fancy {

class String;

  namespace SSL {

    class X509Cert;
    class X509Crl;
    class PKey;

    /**
     * Wrapper around openssl 1.x context.
     */
    class FANCYSSL_API SockCtx : public Base, virtual public Fancy::Sock::DeviceFactory
    {
    public:
    	typedef Fancy::ObjPtr<SockCtx> ptr_t;

    	// the Fancy::SSL::Method enum allows to set the SSL/TLS version used by Fancy::SockCtx::newInstance()
    	// TLSv11 and higher are only supported if the used openssl library supports them, as indicated by the respective defines
    	enum Method     {
    	  SSLv2,    // Very deprecated, NOT SUPPORTED by Fancy::SockCtx
    	  SSLv3,    // Very deprecated, NOT SUPPORTED by Fancy::SockCtx
    	  SSLv23,   // Very deprecated, makes Fancy::SockCtx use SSLv23_server_method(), i.e. SSL 2/3 only
    	  TLSv1,    // Deprecated, makes Fancy::SockCtx use TLSv1_method(), i.e. TLS 1.0 only
    	  TLSv11,   // Deprecated, makes Fancy::SockCtx use TLSv1_1_method(), i.e. TLS 1.1 only
    	  TLSv12,   // Makes Fancy::SockCtx use TLSv1_2_method(), i.e. TLS 1.2 only
    	  DTLS,     // Propably intended to make Fancy::SockCtx use DTLS_method(), but does NOT seem to be SUPPORTED by Fancy::SockCtx yet
    	  TLSv13,   // Makes Fancy::SockCtx use TLS_method() and SSL_CTX_set_min_proto_version(ctx, TLS1_3_VERSION), i.e. TLS 1.3 only
    	  TLSvFlex  // Recommended. Makes Fancy::SockCtx use TLS_method() but not SSL_CTX_set_*_proto_version(), theoretically 'use best available'.
    	            // Feel free to call SSL_CTX_set_min_proto_version(fancySockCtx->getSSL_CTX(), ...) yourself.
    	};

    	enum Type       { Client, Server };
    	enum PeerPolicy {
    	    /**
    	     * No peer certificate is required on the server.
    	     * On the client side, the server certificate is not checked.
    	     */
    	    NoPeerCert,
    	    /**
    	     * This option is the preferred mode, if the peer should be authenticated.
    	     *
    	     * For Client sockets:
    	     * Require the peer server to provide a certificate AND VERIFY it SNI wise,
    	     * i.e. the server certificate must have the hostname in SAN DNS list
    	     * or as CN if the SAN DNS list is empty.
    	     *
    	     * For Server sockets:
    	     * Require the peer client to provide a certificate but NO VERIFICATION.
    	     */
    	    RequirePeerCert,
    	    /**
    	     * For Client sockets:
    	     * Require the peer server to provide a certificate, NO VERIFICATION.
    	     *
    	     * For Server sockets:
    	     * Require the peer server to provide a certificate, AND VERIFY it..
    	     */
    	    VerifyPeerCert
			/*
			 * FIXME FancyLibs 4 clean up, i.e. use new pseudoconstants and always use SSL_CTX_set_verify
			 *
			 * Currently
			 * - server sockets use SSL_CTX_set_verify in  SockCtx::newInstance(Method m, Type t, PeerPolicy p)
			 * - client sockets use SSL_get_peer_certificate in SockDevice::newInstance(const SockCtx::ptr_t &ctx, const Fancy::Sock::Device::ptr_t &dev)
			 */
    	};

    	// method m is ignored for clients
    	static ptr_t newInstance(Method m, Type t, PeerPolicy p);

    	void setSessionID(const std::string &id);

    	PeerPolicy getPeerPolicy() const;
    	Type getType() const;

    	Fancy::ObjPtr<X509Cert> getCertificate() const;

    	// adds ca to CA chain used for verification
    	void addVerifyCA(const Fancy::ObjPtr<X509Cert> &ca);

    	// adds ca to CA chain used for verification and
    	// send it's DN to the client as a hint for selecting the appropriate certificate.
    	void addClientCA(const Fancy::ObjPtr<X509Cert> &ca);

    	void addVerifyCrl(const Fancy::ObjPtr<X509Crl> &crl);

    	void setCertificate(const Fancy::ObjPtr<X509Cert> &x509, const Fancy::ObjPtr<PKey> &pkey);

    	SSL_CTX *getSSL_CTX() const;

    	// override DeviceFactory

    	/**
    	 * If no setSoRecvTimeout(int > 0) has been issued before, this may run into blocking reads!
    	 */
    	Fancy::Sock::Device::ptr_t connect(const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * If no setSoRecvTimeout(int > 0) has been issued before, the accept() of the created SSL::Sock::Device may run into blocking reads!
    	 * The current implementation still cannot serve new clients while waiting for a bad client to time out.
    	 */
    	Fancy::Sock::Device::ptr_t bind(const Fancy::Sock::Address::ptr_t &addr);

    	/**
    	 * If ms>0, the recv timeout is applied to all new sockets right after raw connect.
    	 */
    	void setSoRecvTimeout(int ms);

    	/**
    	 * Returns the value previously set by setSoRecvTimeout(int).
    	 */
    	int getSoRecvTimeout();

        void setCloseOnExec(bool close_on_exec);
        bool getCloseOnExec() const;

    protected:
        SockCtx(Type t, PeerPolicy _pp, SSL_CTX *_ctx);
        ~SockCtx();

    private:
    	Type type;
    	PeerPolicy pp;
    	Fancy::ObjPtr<X509Cert> my_cert;
    	SSL_CTX *ctx;
    	int factorySoRecvTimeout;
    	int close_on_exec;
    };

    class FANCYSSL_API SockDevice : public Fancy::Sock::Device
    {
    public:
      SockDevice(const SockCtx::ptr_t &ctx, const Fancy::Sock::Device::ptr_t &dev, const Fancy::ObjPtr<X509Cert> &peer, ::SSL *_ssl);

      ~SockDevice();

    public:
    	typedef Fancy::ObjPtr<SockDevice> ptr_t;

    	/**
    	 * Wrap a given socket device.
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param dev The device to be used as SSL transport.
    	 * @return An SSL socket device on top of the given device.
    	 */
    	static ptr_t newInstance (const SockCtx::ptr_t &ctx, const Fancy::Sock::Device::ptr_t &dev);

    	/**
    	 * Connect to the given given address using a given SSL context.
    	 * This method uses close_on_exec=true
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param addr The socket address to connect to.
    	 * @return An SSL socket device connected to the given address.
    	 */
    	static ptr_t connect (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr);

        /**
         * Connect to the given given address using a given SSL context.
         *
         * @param ctx The SSL context to use.
         * @param addr The socket address to connect to.
         * @param close_on_exec The close on exec flag for the socket.
         * @return An SSL socket device connected to the given address.
         */
    	static ptr_t connect (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr, bool close_on_exec);

        /**
         * Connect to the given given address using a given SSL context.
         *
         * @param ctx The SSL context to use.
         * @param addr The socket address to connect to.
         * @param close_on_exec The close on exec flag for the socket.
         * @param soRecvTimeout a recevive timeout in millisecods to set before the physical connect.
         * @return An SSL socket device connected to the given address.
         */
        static ptr_t connect (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr, bool close_on_exec, int soRecvTimeout);

        /**
    	 * Bind to the given given address using a given SSL context.
    	 * This method uses close_on_exec=true
    	 *
    	 * @param ctx The SSL context to use.
    	 * @param addr The socket address to bind to.
    	 * @return An SSL socket device bound to the given address.
    	 */
    	static ptr_t bind (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr);

        /**
        * Bind to the given given address using a given SSL context.
        *
        * @param ctx The SSL context to use.
        * @param addr The socket address to bind to.
        * @param close_on_exec The close on exec flag for the socket.
        * @return An SSL socket device bound to the given address.
        */
    	static ptr_t bind (const SockCtx::ptr_t &ctx, const Fancy::Sock::Address::ptr_t &addr, bool close_on_exe);

       /**
    	 * @return The OpenSSL SSL pointer wrapped by this @c SockDevice instance.
    	 */
    	::SSL *getSSL() const;

    	/**
    	 * @return The SSL context used by this device.
    	 */
    	SockCtx::ptr_t getSockCtx() const;

    	/**
    	 * @return The certificate presented by the peer during the initial handshake or
    	 *         @c null, if we did not receive such a certificate.
    	 */
    	Fancy::ObjPtr<X509Cert> getPeerCert() const;

    	/**
    	 * @return The underlying physical device.
    	 */
    	Fancy::Sock::Device::ptr_t getDevice() const;

    	// reimplemented functions from
    	// Fancy::Channel, Fancy::PollableChannel, Fancy::Sock::Device
    	Fancy::Sock::Device::ptr_t accept();
    	void listen (int backlog);

    	void shutdown(Fancy::ShutdownOption o);
    	void close();

    	int read (void *buf, int count);
    	int write (const void *buf, int count);

    	void flush();

    	bool isOpen() const;

    	void setSoRecvTimeout(int ms);
    	int getSoRecvTimeout() const;

#ifdef WIN32
    	void *getEventHandle();
    	int prePoll(PollEvent *events);
    	int postPoll(PollEvent *events);
#else
    	int getPollFileDescriptor() const;
#endif

    	size_t bytesAvailable() const;

    	Fancy::Sock::Address::ptr_t getAddr() const;

    	std::string toString() const;

    	/**
    	 * @return the key used to attach the peer certificate to the thread local
         *         key in @c putPeerCertificate().
    	 */
    	static Fancy::ObjPtr<Fancy::String> getX509CertKey();

    	/**
    	 * detects if a certtname (certificate hostname, i.e. a CN or subject alternate DNS name) covers an expected hostname.
    	 * certName may start with *, i.e. be a wildcard certificate.
    	 *
    	 * As usual, * covers only on . layer:
    	 *   certtname *.clazzes.org covers hostname www123.clazzes.org but not hostname www.subdomain.clazzes.org
    	 */
    	static bool certnameCoversHostname(const std::string & certname, const std::string & hostname);

    private:
    	SockDevice(const SockDevice &);
    	SockDevice &operator= (const SockDevice &);

    	SockCtx::ptr_t sock_ctx;
    	Fancy::Sock::Device::ptr_t device;
    	Fancy::ObjPtr<X509Cert> peer_cert;
    	::SSL *ssl;
    };

  }
}

#endif // ! FANCYSSL_SOCK_DEVICE_H
