/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL X509 Certificates.
**
*****************************************************************************/

#ifndef FANCYSSL_X509CERT_H
#define FANCYSSL_X509CERT_H

#include <fancysslbase.h>
#include <fancysslasn1time.h>

#include <openssl/opensslv.h>


// fwd decls from openssl.
extern "C" {
  struct x509_st;
  typedef struct x509_st X509;
}

namespace Fancy {

  class List;

  namespace SSL {
    class X509Name;
    class X509Ext;
    class BigNum;
    class ASN1Time;
    class PKey;

    class FANCYSSL_API X509Cert : public Base
    {
    protected:
    	X509Cert ();
    	X509Cert (X509 *_x509);

    	~X509Cert();

    public:
    	typedef ObjPtr<X509Cert> ptr_t;

        /**
          Aquires the given OpenSSL handle and automagically free the handle,
          if the object goes out of scope. The caller must not call \c X509_free
          on the supplied handle.
        */
    	static ptr_t newInstance(X509 *_x509);

    	/**
    	  Reads the certificate from a PEM-encoded string.

       	  If len is -1 then then @c pem is assumed to be null terminated and its
    	  length is determined by @c strlen(pem).

    	  Returns the parsed certificate or throws an exception upon errors.
    	*/
    	static ptr_t newInstance(const char *pem, int len=-1);

    	/**
    	 * Read a private key from a ASN1 block.
    	 * @param asn1 The ASN1 block.
    	 * @param len  The length of @c asn1.
    	 * @return A new @c X509Cert instance.
    	 */
    	static ptr_t newInstance(const unsigned char *asn1, int len);

    	/**
    	 * Generate a new X509 certificate instance, which may be signed lateron.
    	 *
    	 * @param subject the certificate subject.
    	 * @param pubKey The public key of the subject.
    	 * @param serial The serial number of the new certificate.
    	 * @param notBefore The start of the validity of the new certificate.
    	 * @param notAfter The end of the validity of the new certificate.
    	 * @return A X509 certificate wihtout extensions, signature and signer info.
    	 */
    	static ptr_t newUnsignedInstance(const Fancy::ObjPtr<X509Name> &subject,
    			                         const Fancy::ObjPtr<PKey> &pubKey,
    			    			         const Fancy::ObjPtr<BigNum> &serial,
    			                         const Fancy::ObjPtr<ASN1Time> &notBefore,
    			                         const Fancy::ObjPtr<ASN1Time> &notAfter);
    	/**
    	 * Add an X509 extension to the certificate prior to signing the certificate.
    	 * @param ext The extension to be added.
    	 */
    	void addExtension(const Fancy::ObjPtr<X509Ext> &ext);

    	/**
    	 * Sign the certificate using the given key.
    	 * @param signer THe signer certificate.
    	 * @param signerKey The private key of the signer.
    	 */
    	void sign(const ptr_t &signer,
                  const Fancy::ObjPtr<PKey> &signerKey);

        /**
            Reads a certificate from a PEM-encoded file.

            Returns the parsed certificate or throws an exception upon errors.
          */
        static ptr_t readFile(const char *file);


        /**
            Reads a certificate chain from a PEM-encoded file, which contains a
            list of certificates.

            Returns the parsed certificates or throws an exception upon errors.
          */
        static Fancy::ObjPtr<Fancy::List> readChainFile(const char *file);

        bool operator == (const X509Cert &a) const;
        bool operator != (const X509Cert &a) const;
        bool operator < (const X509Cert &a) const;

        /**
        Compare a X509 Certificate with the X509 of this object
        return -1,0 or 1 if this certificate is smaller than, equal to or greater than
        the certificate @c a.
        */
        int compare (const X509Cert &a) const;

        bool lessThan(const Fancy::Object &o) const;
        bool equals(const Fancy::Object &o) const;

        ptr_t clone() const;

        /**
           Returns a duplicate the OpenSSL handle of this object. The caller is
           responsible for calling X509_free on the returned handle.
         */
        X509 *dup() const;

        /**
           Returns the OpenSSL handle of this object. THe caller must not call
           X509_free on the returned handle.
         */
        X509 *getX509 () const;

#if OPENSSL_VERSION_NUMBER >= 0x10100000L
        /**
         * Returns the 'notBefore' field of the certificate.
         *
         * Note: Under OpenSSL 1.1.0 this function must be used instead of
         *       'getX509()->cert_info->notBefore'
         */
        const ASN1_TIME* getNotBefore() const;

        /**
         * Returns the 'notAfter' field of the certificate.
         *
         * Note: Under OpenSSL 1.1.0 this function must be used instead of
         *       'getX509()->cert_info->notAfter'
         */
        const ASN1_TIME* getNotAfter() const;
#endif

        std::string getPEMString() const;

        std::string toString() const;

        Fancy::ObjPtr<X509Name> getSubjectDN () const;
        Fancy::ObjPtr<X509Name> getIssuerDN () const;
        Fancy::ObjPtr<BigNum> getSerial () const;

        /**
         * Return a list of subject alternative names with type DNS name, as Fancy::String::ptr_t instances
         */
        Fancy::ObjPtr<Fancy::List> getAlternativeDnsNames() const;

        Fancy::ObjPtr<PKey> getPublicKey() const;

        // HINT: Also take a look at fancyssl-tools' FancySSLToolsUtils et. al.
    private:
    	X509Cert (const X509Cert &a);
    	X509Cert &operator= (const X509Cert &a);
    	X509 *x509;
    };
  }
}

#endif
